# -*- coding: utf-8 -*-

from django.db import models
import datetime


class MigrationHistory(models.Model):

    app_name = models.CharField(max_length=255)
    migration_name = models.CharField(max_length=255)
    applied_on = models.DateTimeField(default=datetime.datetime.now)


    def __str__(self):
        return "<%s: %s>" % (self.app_name, self.migration_name)



import os
import sys


from django.utils import importlib


class Migrations(list):

    def __init__(self, app_name):
        self._app_name = app_name
        migrations_dir = self._ensure_migrations_dir(app_name)
        self._load_migrations(migrations_dir)


    def migrations_module(self):
        return self._app_name + '.appmigrations'


    def migrations_dir(self):
        module_path = self.migrations_module()
        try:
            module = importlib.import_module(module_path)
        except ImportError:
            try:
                parent = importlib.import_module(".".join(module_path.split(".")[:-1]))
            except ImportError:
                raise
            else:
                return os.path.join(os.path.dirname(parent.__file__), module_path.split(".")[-1])
        else:
            return os.path.dirname(module.__file__)


    def _ensure_migrations_dir(self, app_name):
        migrations_dir = self.migrations_dir()
        if not os.path.isdir(migrations_dir):
            os.mkdir(migrations_dir)
        init_path = os.path.join(migrations_dir, "__init__.py")

        if not os.path.isfile(init_path):
            open(init_path, "w").close()
        return migrations_dir


    def _load_migrations(self, migrations_dir):
        filenames = []
        for root, dirs, filenames in os.walk(migrations_dir):
            self.extend([f.split('.')[0] for f in filenames \
                            if f.endswith('.py') and not f.startswith('__init__')])

    def next_filename(self, name):
        highest_number = 0
        for filename in self:
            try:
                number = int(filename.split("_")[0])
                highest_number = max(highest_number, number)
            except ValueError:
                pass
        return "%04i_%s.py" % (highest_number + 1, name)







import re
import os

from django.core.management.base import BaseCommand
from django.db import models
from django.core.exceptions import ImproperlyConfigured

from south_central.migration import Migrations


class Command(BaseCommand):

    def handle(self, app, name, *args, **kwargs):
        if re.search('[^_\w]', name) and name != '-':
            self.error('Migration names should contain only alphanumeric characters and underscores.')

        try:
            app_module = models.get_app(app)
        except ImproperlyConfigured:
            print("There is no enabled application matching '%s'." % app)
            return

        migrations = Migrations(app)
        new_filename = migrations.next_filename(name)

        with open(os.path.join(migrations.migrations_dir(), new_filename), "w") as fp:
            fp.write(MIGRATION_TEMPLATE)


MIGRATION_TEMPLATE = """# -*- coding: utf-8 -*-


class Migration(object):

    def apply(self):
        pass"""



from django.core.management.base import BaseCommand
from django.utils import importlib

from south_central.migration import Migrations
from south_central.models import MigrationHistory


def to_apply(all_migrations, applied):
    return [m for m in all_migrations if not m in applied]


def migration_module(basename, module_name):
    return '.'.join([basename, module_name])


def import_migration_module(migrations_module, migration):
    module = migration_module(migrations_module, migration)
    return __import__(module, {}, {}, ['Migration'])


class Command(BaseCommand):

    def handle(self, app, *args, **kwargs):
        app_migrations = Migrations(app)
        migrations_module = app_migrations.migrations_module()

        applied = MigrationHistory.objects.filter(app_name=app) \
                    .values_list('migration_name', flat=True).order_by('applied_on')
        workplan = to_apply(app_migrations, applied)

        for m in workplan:
            module = import_migration_module(migrations_module, m)
            module.Migration().apply()
            MigrationHistory.objects.create(app_name=app, migration_name=m)
